'use strict';

var react = require('react');
var ReactDOM = require('react-dom');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var ReactDOM__default = /*#__PURE__*/_interopDefaultLegacy(ReactDOM);

const isDev = process.env.NODE_ENV === 'development';
const isObject = (x) => Object.prototype.toString.call(x) === '[object Object]';
function reox(state) {
    if (isDev && !isObject(state))
        throw new Error('object required');
    const _store = {};
    Object.keys(state).forEach((key) => {
        if (typeof state[key] === 'function')
            return;
        const _listeners = new Set();
        _store[key] = {
            subscribe: (listener) => {
                _listeners.add(listener);
                return () => _listeners.delete(listener);
            },
            getSnapshot: () => state[key],
            setSnapshot: (val) => {
                if (val === state[key])
                    return;
                state[key] = val;
                ReactDOM__default["default"].unstable_batchedUpdates?.(() => _listeners.forEach((listener) => listener()));
            },
            useSnapshot: () => {
                return react.useSyncExternalStore(_store[key].subscribe, _store[key].getSnapshot);
            },
        };
    });
    return new Proxy(state, {
        get: (_, key) => {
            try {
                return _store[key].useSnapshot();
            }
            catch (e) {
                return state[key];
            }
        },
        set: (_, key, val) => {
            _store[key].setSnapshot(val);
            return true;
        },
    });
}

module.exports = reox;
